<?php

namespace Wp618\File;

if ( !defined( 'ABSPATH' ) ) exit;

//
class Setup
{
    //
    private $app;

    //
    public function __construct(){
        $this->app = App::getInstance();
    }

    //
    public function load_setup(){
        if($this->app->get_is_actived()){
            if(!empty($_POST)){
                $this->db_show();
            }else{
                $option = 'confirm';
                $data = $this->get_setup_data($option);
                $this->app->display($option, $data);
            }
        }else{
            $action = get_action('app_active');
            $data = [
                'action' => $action,
                'nonce' => \wp_create_nonce($action),
                'option' => 'none',
            ];

            $this->app->display('active', $data);
        }
    }

    //
    private static function fs_get_file_rec_list($path, $is_recursion = false, $is_init = true){
        static $list = [];
        if($is_init){
            $list = [];
        }

        $path = str_replace('\\', '/', $path);

        if(!file_exists($path)){
            return $list;
        }

        $file_list = scandir($path);

        foreach ($file_list as $file){
            if(in_array($file, ['.', '..'])){
                continue;
            }

            $item = [
                'file' => $file,
                'path' => $path . '/' . $file,
            ];

            if($is_recursion && is_dir($item['path'])){
                self::fs_get_file_rec_list($item['path'], $is_recursion, false);
                continue;
            }

            $list[] = $item;
        }

        return $list;
    }

    //
    public function db_show(){
        $files = self::fs_get_file_rec_list(ABSPATH, true);
        $total = 0;
        $total_file_cnt = count($files);

        $parts = [
            ['inc' => '/wp-admin/', 'size' => 0, 'file_cnt' => 0],
            ['inc' => '/wp-content/uploads/', 'size' => 0, 'file_cnt' => 0],
            ['inc' => '/wp-content/', 'size' => 0, 'file_cnt' => 0],
            ['inc' => '/wp-includes/', 'size' => 0, 'file_cnt' => 0],
        ];

        $other_size = 0;
        $other_file_cnt = 0;

        foreach ($files as $item){
            $size = filesize($item['path']);
            $total += $size;

            foreach ($parts as $k => $part){
                if(stripos($item['path'], $part['inc']) === false){
                    continue;
                }

                // 命中
                $parts[$k]['size'] += $size;
                $parts[$k]['file_cnt'] += 1;
                continue 2;
            }

            $other_size += $size; // 其他
            $other_file_cnt += 1;
        }

        $list = [];
        foreach ($parts as $part){
            $value = self::get_value_txt($part['file_cnt'], $part['size']);
            $list[] = ['name' => $part['inc'], 'value' => $value];
        }

        $value = self::get_value_txt($other_file_cnt, $other_size);
        $list[] = ['name' => '其他', 'value' => $value];

        $value = self::get_value_txt($total_file_cnt, $total);
        $list[] = ['name' => '总数', 'value' => $value];

        $action = get_action('delete_more_log');
        $nonce = \wp_create_nonce($action);

        $this->app->display('log', [
            'list' => $list,
            'action' => $action,
            'nonce' => $nonce,
        ]);
    }

    //
    private static function get_value_txt($file_cnt, $size){
        return $file_cnt . ' 个文件, ' . number_format($size) . ' 字节';
    }

    //
    private function get_setup_data($option){
        $config = get_app_config();
        $set = $this->get_render_items($option, $config);

        $action = get_action('setup_save');

        $data = [
            'action' => $action,
            'nonce' => \wp_create_nonce($action),
            'option' => $option,
            'set' => $set,
        ];

        return $data;
    }

    //
    private function get_render_items($option, $config){
        if(isset($config[$option])){
            return $config[$option];
        }

        $list = [];

        if(isset(Constant::SETUP_FIELDS[$option])){
            foreach (Constant::SETUP_FIELDS[$option] as $name => $val){
                $list[$name] = $val[1];
            }
        }

        return $list;
    }
}