<?php

namespace Wp618\File;

if ( !defined( 'ABSPATH' ) ) exit;

require_once __DIR__ . '/../config/Constant.php';
require_once __DIR__ . '/../helpers/normal.php';
require_once __DIR__ . '/../helpers/wps.php';
require_once __DIR__ . '/../library/Filter.php';
require_once __DIR__ . '/../library/Request.php';
require_once __DIR__ . '/../library/Time.php';
require_once __DIR__ . '/../library/Paging.php';
require_once __DIR__ . '/../system/AppSet.php';
require_once __DIR__ . '/../controller/Setup.php';

//
class App{
    private static $instance;
    public $request;
    public $filter;
    private $appSet;
    private $acvited_data = [];
    private $app_data = [];
    private $app_file = '';

    //
    private function __construct() {}

    //
    public function get_app_item($name){
        if(isset($this->app_data[$name])){
            return $this->app_data[$name];
        }

        return null;
    }

    //
    private function get_app_data(){
        $path = dirname(dirname(__DIR__));
        $app_type = basename(dirname($path));
        $is_theme = $app_type == 'themes';

        $app_dir = $path . '/';
        $app_uri = str_replace('\\', '/', substr($path, strlen($_SERVER['DOCUMENT_ROOT']))) . '/';

        $style = file_get_contents($this->app_file);

        $app_name = basename($path);

        preg_match('#Version:(.*?)\n#i', $style, $match);
        $app_version = isset($match[1]) ? trim($match[1]) : '0.0.1';

        $prefix = 'vo_' . str_ireplace(['vong-', '-'], ['', '_'], strtolower(basename($path))) . '_';

        $this->app_data = [
            'app_prefix' => $prefix,
            'app_type' => $app_type,
            'is_theme' => $is_theme,
            'app_name' => $app_name,
            'app_version' => $app_version,
            'app_dir' => $app_dir,
            'app_uri' => $app_uri,
        ];
    }

    //
    public function get_is_actived(){
        return $this->acvited_data['is_actived'];
    }

    //
    private function read_active_data(){
        if(empty($this->acvited_data)){
            $key = App::getInstance()->get_app_item('app_prefix') . 'actived';

            $this->acvited_data = get_option($key) ? : [];

            if(empty($this->acvited_data['is_actived'])){
                $this->acvited_data['is_actived'] = false;
            }
        }
    }

    //
    public function display($view, $data){
        $view = preg_replace('#[^\w\-]#', '', $view);
        $view = preg_replace('#\.+#', '.', $view);

        extract($data);

        include __DIR__ . '/../view/' . $view . '.php';
    }

    //
    private function load_library(){
        $this->request = new Request;
        $this->filter = new filter;
    }

    //
    private function load_model(){
        // model
        if(!empty(Constant::MODEL_LIST)){
            require_once __DIR__ . '/Model.php';
        }
        foreach (Constant::MODEL_LIST as $model){
            $name = ucfirst($model);
            require_once __DIR__ . '/../model/' . $name . '.php';
            $name = '\\' . __NAMESPACE__ . '\\' . $name;
            $this->$model = new $name();
        }
    }

    //
    private function load_controller(){
        $this->setup = new Setup();
    }

    //
    private function load_func($file){
        $this->app_file = $file;
        $this->get_app_data();
        $this->read_active_data();
        $this->load_library();
        $this->load_model();
        $this->load_controller();
        $this->load_hook();

        // install
        if(Constant::INSTALL_CALL){
            require_once __DIR__ . '/../install/Plugin_install.php';
            new Plugin_install($this->app_file);
        }

        //
        if(!empty(Constant::MENU_SHOW)){
            require_once __DIR__ . '/../menu/Menu.php';
            new Menu();
        }

        //
        if(!empty(Constant::AJAX_CALL)){
            require_once __DIR__ . '/../ajax/Ajax.php';
            new Ajax();
        }

        $this->appSet = new appSet();
    }

    //
    private function load_hook(){
        // action
        if(Constant::ACTION_FILTER_CHECKED['action']){
            require_once __DIR__ . '/../hook/Action.php';
            new Action();
        }

        // action_api
        if(Constant::ACTION_FILTER_CHECKED['action_api']){
            require_once __DIR__ . '/../hook/Action_api.php';
            new Action_api();
        }

        // filter
        if(Constant::ACTION_FILTER_CHECKED['hook_filter']){
            require_once __DIR__ . '/../hook/Hook_filter.php';
            new Hook_filter();
        }
    }

    //
    public static function init($file){
        self::getInstance($file);
    }

    //
    public static function getInstance($file = '') {
        if (!isset(self::$instance)) {
            self::$instance = new self();
            self::$instance->load_func($file);
        }

        return self::$instance;
    }

}
