<?php

namespace Wp618\File;

if ( !defined( 'ABSPATH' ) ) exit;

//
function get_image_width_height($file){
    $ret = [
        'width' => 0,
        'height' => 0,
        'wxh' => '',
    ];

    if(!file_exists($file)){
        return $ret;
    }

    $info = getimagesize($file);

    if(!empty($info[0]) && !empty($info[1])){
        $ret['width'] = $info[0];
        $ret['height'] = $info[1];
        $ret['wxh'] = $info[0] . 'x' . $info[1];
    }

    return $ret;
}

// --- 文件处理开始 ---
//
function get_file_ext($file, $is_with_dot = false){
    preg_match('#\.(\w{1,6})$#', rtrim(strtolower($file)), $match);

    if(empty($match)){
        return '';
    }

    return $is_with_dot ? strtolower($match[0]) : strtolower($match[1]);
}

//
function delete_file_ext($file){
    return preg_replace('#\.\w+$#', '', $file);
}

//
function read_file($path){
    return file_exists($path) ? file_get_contents($path) : '';
}

//
function write_file($file, $content, $is_append = false){
    check_file($file);

    $flags = $is_append ? FILE_APPEND : 0;

    return file_put_contents($file, $content, $flags);
}

//
function check_file($path, $is_file = true){
    $dir = $path;
    if($is_file){
        $dir = dirname($path);
    }

    if(!file_exists($dir)){
        mkdir($dir, 0755, true);
    }

    return true;
}
// --- 文件处理结束 ---

// --- html 处理结束 ---
//
function html_base($tag = '', $attrs = [], $content = '', $is_close = false)
{
    if (strlen($tag) < 1) {
        return '';
    }

    $tag = strtolower($tag);
    $html = "<{$tag}";

    foreach ($attrs as $key => $val){
        $val = str_replace('"', '&quot;', $val);
        $html .= ' ' . $key . '=' . '"' . $val . '"';
    }

    $html .= '>';

    if(strlen($content) > 0){
        $html .= $content;
        $is_close = true;
    }

    if ($is_close) {
        $html .= '</' . $tag . '>';
    }

    return $html;
}

//
function html_a($href, $title = '', $is_blank = false)
{
    $arr = ['href' => $href];

    if ($is_blank) {
        $arr['target'] = '_blank';
    }

    return html_base('a', $arr, $title, true);
}

//
function html_css_link($href)
{
    return '<link rel="stylesheet" href="' . $href . '">';
}

//
function html_script($src)
{
    return '<script src="' . $src . '"></script>';
}

//
function html_no_referrer()
{
    return '<meta name="referrer" content="no-referrer">';
}

//
function html_viewport()
{
    return '<meta name="viewport" content="width=device-width, initial-scale=1">';
}

//
function html_img($src, $alt = '')
{
    $attrs = ['src' => $src];

    if(strlen($alt) > 0){
        $attrs['alt'] = $alt;
    }

    return html_base('img', $attrs, '', false);
}

//
function html_textarea($attrs, $content)
{
    $content = htmlspecialchars( $content, ENT_QUOTES);

    return html_base('textarea', $attrs, $content, true);
}

//
function html_space($num = 2)
{
    $str = str_repeat('&nbsp', $num);

    return $str;
}

// --- html 处理结束 ---

// --- 分页处理开始 ---
//
function get_page_count($total, $limit){
    return ceil($total / $limit);
}
//
function get_offset($page, $limit){
    return max(0, ($page - 1) * $limit);
}

//
function get_range_start($page, $limit){
    return get_offset($page, $limit) + 1;
}

//
function get_range_end($page, $limit, $total){
    return min($limit * $page, $total);
}

// --- 分页处理结束 ---

// --- 字符串处理开始 ---

//不区分大小写的字符切割
function isplit($sp, $str)
{
    return explode($sp, str_ireplace(strtoupper($sp), $sp, $str));
}

// 字符截取 不区分大小写，接受负数
function str_cut($str, $flag_str, $n = 0, $no_crlf = 1, $no_html = 1, $trim = 1)
{
    if ($str === '' || stripos($str, $flag_str) === false) {
        return '';
    }

    $n = (int)$n;
    $arr = isplit($flag_str, $str);

    if($n < 0){
        $n = count($arr) + $n;
    }

    if(!isset($arr[$n])){
        return '';
    }

    $res = $arr[$n];

    // 参数合并
    if (strlen($no_crlf) == 3) {
        $no_html = substr($no_crlf, 1, 1);
        $trim = substr($no_crlf, 2, 1);
        $no_crlf = substr($no_crlf, 0, 1);
    }

    if ($no_crlf > 0) {
        $res = str_replace(["\r", "\n"], '', $res); // 删除换行符
    }

    if ($no_html > 0) {
        $res = strip_tags($res); //删除html
    }

    if ($trim > 0) {
        $res = trim($res);
    }

    return $res;
}

function str_cut2($str, $flag_str1, $flag_str2, $n = 0, $x = 0, $no_crlf = 1, $no_html = 1, $trim = 1)
{
    $str = str_cut($str, $flag_str1, $n, '000');
    $str = str_cut($str, $flag_str2, $x, $no_crlf, $no_html, $trim);

    return $str;
}

function str_cut3($str, $flag_str1, $flag_str2, $flag_str3, $n = 0, $x = 0, $y = 0, $no_crlf = 1, $no_html = 1, $trim = 1)
{
    $str = str_cut($str, $flag_str1, $n, '000');
    $str = str_cut($str, $flag_str2, $x, '000');
    $str = str_cut($str, $flag_str3, $y, $no_crlf, $no_html, $trim);

    return $str;
}

//
function replace_quote($val){
    return str_replace('"', '&quot;', $val);
}

// 是否包含中文
function is_contain_chinese($str)
{
    return (bool) preg_match('/[\x{4e00}-\x{9fa5}]/u', $str);
}

//
function safe_key( $key ) {
    $sanitized_key = '';

    if ( is_scalar( $key ) ) {
        $sanitized_key = strtolower( $key );
        $sanitized_key = preg_replace( '/[^a-z0-9_\-]/', '', $sanitized_key );
    }

    return $sanitized_key;
}
// --- 字符串处理结束 ---

//功能：返回js代码
function js_code($str = '')
{
    return '<script>' . $str . '</script>';
}

//功能：定时跳转 时间默认为：秒
function js_location($url = '', $t = 0, $type = 's')
{
    $second = intval($t);

    if ($type === 's') {
        $second *= 1000;
    }

    $js = '
			setTimeout(function(){
				window.location.href = "' . $url . '";
			}, ' . $second . ')
		';

    return js_code($js);
}

//
function get_content_add_para($content){
    if(stripos($content, '</p>') !== false){
        return $content;
    }

    $arr = explode("\n", $content);
    $content = '';

    foreach ($arr as $line){
        $line = trim($line);

        if($line == ''){
            continue;
        }

        $content .= '<p>' . $line . '</p>';
    }

    return $content;
}

//
function get_client_ip(){
    if(isset($_SERVER['HTTP_X_FORWARDED_FOR'])){
        $ip = $_SERVER['HTTP_X_FORWARDED_FOR'];
    }else{
        $ip = $_SERVER['REMOTE_ADDR'];
    }

    return preg_replace('#[^\d.]#', '', $ip);
}